@@ -5,6 +5,7 @@ import tensorflow.keras as keras
 from tensorflow.keras import layers
 from tensorflow.keras import activations
 from . import load
+from . import addons
 import tfbox.utils.helpers as h
 #
 # CONSTANTS
@@ -29,7 +30,7 @@ UPSAMPLE_ERROR=(
 def get(key,activation=False):
     key=h.snake(key)
     if activation:
-        act=ACTIVATIONS.get(key)
+        act=ACTIVATIONS.get(key.lower())
         if not act:
             act=getattr(activations,h.camel(key))
         return act
@@ -54,7 +55,7 @@ def get_activation(act,**config):
         if act is True:
             act=DEFAULT_ACTIVATION
         if isinstance(act,str):
-            act=ACTIVATIONS[act.lower()]
+            act=get(act,activation=True)
         act=act(**config)
     return act
 
@@ -172,8 +173,12 @@ def _valid_key(key):
 # GENERAL BLOCKS 
 #
 class Conv(keras.Model):
-    """ Conv-BatchNorm-Activation-Dropout
+    """ Conv-Norm-Activation-Dropout
     """
+    BATCH_NORM='batch'
+    GROUP_NORM='group'
+
+
     #
     # PUBLIC
     #
@@ -182,7 +187,8 @@ class Conv(keras.Model):
             kernel_size=3,
             padding='same',
             seperable=False,
-            batch_norm=True,
+            norm=BATCH_NORM,
+            norm_config={},
             act=True,
             act_config={},
             dilation_rate=1,
@@ -215,7 +221,7 @@ class Conv(keras.Model):
             strides=strides,
             name=self._layer_name(cname),
             **conv_config)
-        self.bn=self._batch_norm(batch_norm)
+        self.norm=self._norm(norm,norm_config)
         self.act=self._activation(act,act_config)
         self.do=self._dropout(dropout,dropout_config)
         self.act_last=act_last
@@ -225,12 +231,12 @@ class Conv(keras.Model):
     def __call__(self,x,training=False):
         x=self.conv(x)
         if self.act_last:
-            if self.bn: x=self.bn(x)
+            if self.norm: x=self.norm(x)
             if training and self.do: x=self.do(x)
             if self.act: x=self.act(x)
         else:
             if self.act: x=self.act(x)
-            if self.bn: x=self.bn(x)
+            if self.norm: x=self.norm(x)
             if training and self.do: x=self.do(x)         
         return x
 
@@ -242,10 +248,18 @@ class Conv(keras.Model):
         return layer_name(self.block_name,name,named=self.named_layers)
 
 
-    def _batch_norm(self,batch_norm):
-        if batch_norm:
-            bn=layers.BatchNormalization(name=self._layer_name('batch_norm'))
-            return bn      
+    def _norm(self,norm,norm_config):
+        if norm==Conv.BATCH_NORM:
+            norm=layers.BatchNormalization(
+                name=self._layer_name('batch_norm'),
+                **norm_config)
+        elif norm==Conv.GROUP_NORM:
+            norm=addons.GroupNormalization(
+                name=self._layer_name('group_norm'),
+                **norm_config)
+        else:
+            norm=False
+        return norm      
 
 
     def _activation(self,act,config):
@@ -291,7 +305,8 @@ class Stack(keras.Model):
             residual_act=True,
             seperable_residual=False,
             seperable=False,
-            batch_norm=True,
+            norm=True,
+            norm_config={},
             act=True,
             act_config={},
             input_dilation_rate=1,
@@ -329,7 +344,8 @@ class Stack(keras.Model):
             max_pooling,
             seperable=seperable,
             seperable_residual=seperable_residual,
-            batch_norm=batch_norm,
+            norm=norm,
+            norm_config=norm_config,
             padding=padding,
             act_config=act_config,
             **conv_config)
@@ -502,7 +518,6 @@ class Group(keras.Model):
             residual=True,
             residual_act=True,
             seperable=False,
-            batch_norm=True,
             act=True,
             act_config={},
             act_last=False,
@@ -798,7 +813,8 @@ class SegmentClassifier(keras.Model):
             filters_list=None,
             kernel_size=3,
             kernel_size_list=None,
-            output_batch_norm=True,
+            output_norm=True,
+            output_norm_config={},
             output_act=True,
             output_act_config={},
             seperable_preclassification=False,
@@ -833,7 +849,8 @@ class SegmentClassifier(keras.Model):
             self.classifier=Conv(
                     filters=filters_list[-1],
                     kernel_size=kernel_size_list[-1],
-                    batch_norm=output_batch_norm,
+                    norm=output_norm,
+                    norm_config=output_norm_config,
                     act=False,
                     name=self.block_name,
                     named_layers=self.named_layers)
@@ -911,6 +928,7 @@ BLOCKS={
 ACTIVATIONS={
     'relu': layers.ReLU,
     'sigmoid': activations.sigmoid,
+    'swish': addons.Swish,
     'softmax': layers.Softmax
 }
 STACK_BTYPE='stack'
